/* --- Copyright University of Sussex 2001. All rights reserved. ----------
 > File:            $popvision/lib/oldarray.p
 > Purpose:         Avoid garbage collection when getting temporary arrays
 > Author:          David S Young, Nov 15 1994 (see revisions)
 > Documentation:   HELP * OLDARRAY
 */

compile_mode:pop11 +strict;

section;

define /*lconstant*/ boundsize(bounds) -> n;
    lvars bounds, n = 1;
    lvars x0, x1;
    until bounds == [] do
        dest(dest(bounds)) -> (x0, x1, bounds);
        n * (x1 - x0 + 1) -> n
    enduntil
enddefine;

define oldanyarray(tag, bounds, key) -> arr;
    lvars tag, bounds, init = false, key, arr;
    lconstant
        propsize1 = 5,
        propsize2 = 20,
        typestore = newproperty([], propsize1, false, "perm");

    ;;; See if an initialiser given
    unless bounds.islist and bounds /== [] and back(bounds) /== [] then
        (tag, bounds) -> (tag, bounds, init)
    endunless;

    if tag then     ;;; act like newanyarray if tag is false

        ;;; Get property for this type of array
        lvars arraystore;
        unless typestore(key) ->> arraystore then
            newproperty([], propsize2, false, "tmpval") ->> arraystore
                -> typestore(key)
        endunless;

        ;;; Get array for this tag
        unless (arraystore(tag) ->> arr) and boundslist(arr) = bounds
        and arr.isarray_by_row == poparray_by_row and not(init) then
            if arr and length(arrayvector(arr)) >= boundsize(bounds) then
                ;;; Create an array on top of the existing arrayvector.
                ;;; Note - this new array is not stored in the property, so that
                ;;; the entry can be garbage collected. This is desirable when
                ;;; a reduction in size has occurred and memory is short.
                newanyarray(bounds, if init then init endif, arr) -> arr
            else
                ;;; create a completely new array
                newanyarray(bounds, if init then init endif, key)
                    ->> arr -> arraystore(tag)
            endif
        endunless

    else

        newanyarray(bounds, if init then init endif, key) -> arr

    endif
enddefine;

define oldarray with_nargs 2;
    oldanyarray(vector_key)
enddefine;

endsection;

/* --- Revision History ---------------------------------------------------
--- David Young, Oct 26 2001
        Changed to act like newarray if the tag is false.
--- David S Young, Sep  4 1997
        Added check to see that poparray_by_row is respected.
 */
